@@ -6,44 +6,15 @@
 package require  sak::test::shell
 package require  sak::registry
 package require  sak::animate
+package require  sak::color
+# TODO: Rework this package to use the sak::feedback package
 
 getpackage textutil::repeat textutil/repeat.tcl
 getpackage fileutil         fileutil/fileutil.tcl
 
 namespace eval ::sak::test::run {
     namespace import ::textutil::repeat::blank
-}
-
-if {$::tcl_platform(platform) == "windows"} {
-    # No ansi colorization on windows
-    namespace eval ::sak::test::run {
-	variable n
-	foreach n {cya yel whi mag red rst} {
-	    proc $n {} {return ""}
-	}
-	unset n
-    }
-} else {
-    getpackage term::ansi::code::attr term/ansi/code/attr.tcl
-    getpackage term::ansi::code::ctrl term/ansi/code/ctrl.tcl
-
-    ::term::ansi::code::ctrl::import ::sak::test::run sda_bg* sda_reset
-
-    namespace eval ::sak::test::run {
-	variable s
-	variable n
-	foreach {s n} {
-	    sda_bgcyan    cya
-	    sda_bgyellow  yel
-	    sda_bgwhite   whi
-	    sda_bgmagenta mag
-	    sda_bgred     red
-	    sda_reset     rst
-	} {
-	    rename $s $n
-	}
-	unset s n
-    }
+    namespace import ::sak::color::*
 }
 
 # ###
@@ -112,11 +83,13 @@ proc ::sak::test::run::Do {cv modules} {
     }
 
     if {$alog} {
-	variable logext [open $config(stem).log      w]
-	variable logsum [open $config(stem).summary  w]
-	variable logfai [open $config(stem).failures w]
-	variable logski [open $config(stem).skipped  w]
-	variable lognon [open $config(stem).none     w]
+	variable logext [open $config(stem).log         w]
+	variable logsum [open $config(stem).summary     w]
+	variable logfai [open $config(stem).failures    w]
+	variable logski [open $config(stem).skipped     w]
+	variable lognon [open $config(stem).none        w]
+	variable logerd [open $config(stem).errdetails  w]
+	variable logfad [open $config(stem).faildetails w]
     } else {
 	variable logext stdout
     }
@@ -276,21 +249,17 @@ proc ::sak::test::run::Process {pipe} {
 	if {[eof  $pipe]} break
 	if {[gets $pipe line] < 0} break
 	if {$alog || $araw} {puts $logext $line ; flush $logext}
+	set rline $line
 	set line [string trim $line]
 	if {[string equal $line ""]} continue
 	Host;	Platform
 	Cwd;	Shell
-	Tcl;	Match||Skip||Sourced
+	Tcl
 	Start;	End
 	Module;	Testsuite
 	NoTestsuite
 	Support;Testing;Other
 	Summary
-
-	TestStart
-	TestSkipped
-	TestPassed
-	TestFailed
 	CaptureFailureSync
 	CaptureFailureCollectBody
 	CaptureFailureCollectActual
@@ -298,10 +267,16 @@ proc ::sak::test::run::Process {pipe} {
 	CaptureStackStart
 	CaptureStack
 
+	TestStart
+	TestSkipped
+	TestPassed
+	TestFailed
+
 	SetupError
 	Aborted
 	AbortCause
 
+	Match||Skip||Sourced
 	# Unknown lines are printed
 	if {!$araw} {puts !$line}
     }
@@ -463,6 +438,7 @@ proc ::sak::test::run::Summary {} {
     upvar 2 pass  _pass  ; incr _pass  $p
     upvar 2 skip  _skip  ; incr _skip  $s
     upvar 2 fail  _fail  ; incr _fail  $f
+    upvar 2 err   _err
 
     set t [format %5d $t]
     set p [format %5d $p]
@@ -489,7 +465,10 @@ proc ::sak::test::run::Summary {} {
     switch -exact -- $xstatus {
 	none    {=| "~~ [yel]$st T $t[rst] P $p S $s F $f"}
 	aborted {=| "~~ [whi]$st[rst] T $t P $p S $s F $f"}
-	error   {=| "~~ [mag]$st[rst] T $t P $p S $s F $f"}
+	error   {
+	    =| "~~ [mag]$st[rst] T $t P $p S $s F $f"
+	    incr _err
+	}
 	fail    {=| "~~ [red]$st[rst] T $t P $p S $s [red]F $f[rst]"}
     }
     return -code continue
@@ -573,7 +552,7 @@ proc ::sak::test::run::CaptureFailureSync {} {
 proc ::sak::test::run::CaptureFailureCollectBody {} {
     variable xcollect
     if {$xcollect != 2} return
-    upvar 1 line line
+    upvar 1 rline line
     variable xbody
     if {![string match {---- Result was*} $line]} {
 	variable xbody
@@ -587,7 +566,7 @@ proc ::sak::test::run::CaptureFailureCollectBody {} {
 proc ::sak::test::run::CaptureFailureCollectActual {} {
     variable xcollect
     if {$xcollect != 3} return
-    upvar 1 line line
+    upvar 1 rline line
     if {![string match {---- Result should*} $line]} {
 	variable xactual
 	append   xactual $line \n
@@ -600,11 +579,32 @@ proc ::sak::test::run::CaptureFailureCollectActual {} {
 proc ::sak::test::run::CaptureFailureCollectExpected {} {
     variable xcollect
     if {$xcollect != 4} return
-    upvar 1 line line
+    upvar 1 rline line
     if {![string match {==== *} $line]} {
 	variable xexpected
 	append   xexpected $line \n
     } else {
+	variable alog
+	if {$alog} {
+	    variable logfad
+	    variable xtest
+	    variable xbody
+	    variable xactual
+	    variable xexpected
+
+	    puts  $logfad "==== [lrange $xtest end-1 end] FAILED ========="
+	    puts  $logfad "==== Contents of test case:\n"
+	    puts  $logfad $xbody
+
+	    puts  $logfad "---- Result was:"
+	    puts  $logfad [string range $xactual 0 end-1]
+
+	    puts  $logfad "---- Result should have been:"
+	    puts  $logfad [string range $xexpected 0 end-1]
+
+	    puts  $logfad "==== [lrange $xtest end-1 end] ====\n\n"
+	    flush $logfad
+	}
 	set xcollect 0
 	#sak::registry::local set $xtest Body     $xbody
 	#sak::registry::local set $xtest Actual   $xactual
@@ -630,7 +630,7 @@ proc ::sak::test::run::Aborted {} {
 proc ::sak::test::run::AbortCause {} {
     upvar 1 line line
     if {
-	![string match {Requiring at least *} $line] &&
+	![string match {Requiring *} $line] &&
 	![string match {Error in *} $line]
     } return ; # {}
     variable xfile
@@ -659,7 +659,16 @@ proc ::sak::test::run::CaptureStack {} {
     } else {
 	set xstackcollect 0
 	variable xfile
+	variable alog
 	#sak::registry::local set $xfile Stacktrace $xstack
+	if {$alog} {
+	    variable logerd
+	    puts  $logerd "[lindex $xfile end] StackTrace"
+	    puts  $logerd "========================================"
+	    puts  $logerd $xstack
+	    puts  $logerd "========================================\n\n"
+	    flush $logerd
+	}
     }
     return -code continue
 }
