@@ -31,20 +31,24 @@ array set guide {}
 # @@ Registration START
 
 Exclude calendar
+Exclude exif
 
 #       name         pkg   doc   example
 Module  aes         _tcl  _man  _null
+Module  amazon-s3   _tcl  _man  _null
 Module  asn         _tcl  _man  _null
 Module  base64      _tcl  _man  _null
 Module  base32      _tcl  _man  _null
 Module  bee         _tcl  _man  _null
 Module  bench       _tcl _null  _null
-Module  bibtex      _tcl  _man  _null
+Module  bibtex      _tcl  _man  _exa
 Module  blowfish    _tcl  _man  _null
+Module  cache       _tcl  _man  _null
 Module  calendar     _tci _man  _null
 Module  cmdline     _tcl  _man  _null
 Module  comm        _tcl  _man  _null
 Module  control      _tci _man  _null
+Module  coroutine   _tcl _null  _null
 Module  counter     _tcl  _man  _null
 Module  crc         _tcl  _man  _null
 Module  csv         _tcl  _man _exa
@@ -52,6 +56,9 @@ Module  des         _tcl  _man  _null
 Module  dns          _msg _man _exa
 Module  docstrip    _tcl  _man  _null
 Module  doctools     _doc _man _exa
+Module  doctools2base _tcl _man _null
+Module  doctools2idx  _tcl _man _null
+Module  doctools2toc  _tcl _man _null
 Module  exif        _tcl  _man  _null
 Module  fileutil    _tcl  _man  _null
 Module  ftp         _tcl  _man _exa
@@ -61,7 +68,7 @@ Module  grammar_fa  _tcl  _man  _null
 Module  grammar_me  _tcl  _man  _null
 Module  grammar_peg _tcl  _man  _null
 Module  html        _tcl  _man  _null
-Module  htmlparse   _tcl  _man  _null
+Module  htmlparse   _tcl  _man  _exa
 Module  http        _tcl  _man  _null
 Module  ident       _tcl  _man  _null
 Module  inifile     _tcl  _man  _null
@@ -71,7 +78,8 @@ Module  javascript  _tcl  _man  _null
 Module  jpeg        _tcl  _man  _null
 Module  json        _tcl  _man  _null
 Module  ldap        _tcl  _man _exa
-Module  log          _msg _man  _null
+Module  log          _msg _man  {_exax logger}
+Module  map         _tcl  _man  _null
 Module  mapproj     _tcl  _man _exa
 Module  math         _tci _man _exa
 Module  md4         _tcl  _man  _null
@@ -94,18 +102,21 @@ Module  profiler    _tcl  _man  _null
 Module  rc4         _tcl  _man  _null
 Module  rcs         _tcl  _man  _null
 Module  report      _tcl  _man  _null
+Module  rest        _tcl  _man  _null
 Module  ripemd      _tcl  _man  _null
-Module  sasl        _tcl  _man  _null
+Module  sasl        _tcl  _man  _exa
 Module  sha1        _tcl  _man  _null
+Module  simulation  _tcl  _man  _null
 Module  smtpd       _tcl  _man _exa
 Module  snit        _tcl  _man  _null
 Module  soundex     _tcl  _man  _null
 Module  stooop      _tcl  _man  _null
+Module  stringprep  _tcl  _man  _null
 Module  struct      _tcl  _man _exa
 Module  tar         _tcl  _man  _null
 Module  term         _tcr _man _exa
 Module  textutil     _tex _man  _null
-Module  tie         _tcl  _man  _null
+Module  tie         _tcl  _man  _exa
 Module  tiff        _tcl  _man  _null
 Module  transfer    _tcl  _man  _null
 Module  treeql      _tcl  _man  _null
@@ -113,7 +124,11 @@ Module  uev         _tcl  _man  _null
 Module  units       _tcl  _man  _null
 Module  uri         _tcl  _man  _null
 Module  uuid        _tcl  _man  _null
+Module  virtchannel_base       _tcl _null  _null
+Module  virtchannel_core       _tcl _null  _null
+Module  virtchannel_transform  _tcl _null  _null
 Module  wip         _tcl  _man  _null
+Module  yaml        _tcl  _man  _null
 
 Application  dtplite
 Application  tcldocstrip
